'****************************************************************
'*  Name    : Deviation.BAS                                      *
'*  Author  : [select VIEW...EDITOR OPTIONS]                    *
'*  Notice  : Copyright (c) 2014 [select VIEW...EDITOR OPTIONS] *
'*          : All Rights Reserved                               *
'*  Date    : 21/12/2014                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************
'Simple routine for a high-resolution bargraph on a HD44780 based LCD display.
'In this example, Input A0 is configured to read an input voltage between VSS and VDD 
'(usually 0...5V) and display the relative level as a bargraph where 5V equals full-scale
'reading. 


'The routine works as follows:
' - Prior to the while-wend loop, the LCD screens special character memory is programmed 
'   to enable the high-resolution readout
'
' - First a 10-bit ADC read occurs (value between 0....1023)
'
' - Then the calculations are done:
'       - First the value per segment (SUM byte) is calculated by multiplying the ADC read 
'         value with the maximum number of segments (in this case 16) and then divide it by
'         max read / number of scale lines per segment (1024 / 3)
'       - Then the number of full characters to display (FULL byte) is calculated  
'       - Finally the partial (PARTIAL byte) characters (0, 1 or 2) are calculated by extracting
'         the modulus of the restvalue

' - Finally, the result is displayed:
'       - First print the number of "FULL" characters
'       - Then print the "PARTIAL" character, which either is empty, 1 line or 2 lines
'       - Finally, clear the remaining segments to ensure that no old information still on the display 
'
' - And jump back to the beginning for a "real-time" readout
'
' Mischa van Santen (PA1OKZ) - Netherlands
' VK5DJ additions for a deviation meter developed by Brian VK5VI and me. Facility for calibration.
' note Brian's version will use Ports A.2 for mute, and A.3, A4 the buttons


'Config
;-------------------------------------------------------------------------------
;**** Added by Fuse Configurator ****
; Use the Fuse Configurator plug-in to change these settings

Device = 16F1827

Declare Reminders Off
@ CONFIG_REQ = 0 ; Override Compiler's configuration settings
Asm-
__Config _Config1, 0x09E4 ;FOSC_INTOSC & WDTE_OFF & PWRTE_OFF & MCLRE_ON & CP_OFF & CPD_OFF & BOREN_OFF & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF 
__Config _Config2, 0x1CFF ;WRT_OFF & PLLEN_OFF & STVREN_OFF & BORV_LO & LVP_OFF 
Endasm-
Declare Reminders On

;**** End of Fuse Configurator Settings ****
;-------------------------------------------------------------------------------


                                       
Declare Xtal = 4
OSCCON = %01101010            'set int osc to 4MHz
All_Digital true
Clear
                             
'Declare A/D converter
Declare Adin_Res = 10         ' 10-bit result
Declare Adin_Tad = FRC        ' RC oscillator
Declare Adin_Stime = 50       ' 50us sample time

'setup LCD
Declare LCD_Type 0	          'text type
Declare LCD_DTPin PORTB.4     'assigns data lines to B4..7  
Declare LCD_ENPin PORTB.3     'enable pin
Declare LCD_RSPin PORTB.2     'RS line pin
Declare LCD_Interface 4	      '4 or 8 input line interface
Declare LCD_Lines 2           '2 line display

ADCON0 = %00000001            'enable A/D conversion
ADCON1 = %10110000            'set for right justified
ANSELA = %00000110            'all port a as digital except A.1 (pin 18)
ANSELB = %00000000            'port b as digital

TRISA = %11000110             'A.1 is analog, A.6 and A.7 are buttons    make A.2 analog
TRISB = %00000000             'all outputs


'Variables
Dim Frequency       As Float                  ' frequency by formula
Dim SEGMENTS        As Byte                   ' Holds the number of segments of the display
Dim METER           As Word                   ' Holds A/D value for meter reading
Dim FULL            As Byte                   ' Holds the quantity of "full" characters to print |||
Dim PARTIAL         As Byte                   ' Holds the value 1, 2 or 3 for the partial character | or ||
Dim SUM             As Byte                   ' Holds the result of the sum (METER * 16)/341
Dim VOLTREAD        As Word                   ' whole volts for printout
Dim PARTVOLTS       As Word                   ' first dec for readout
Dim CALIBRATED      As Byte                    ' defines if calibrated  0=none, 1 = calibrated
Dim CountLoop       As Byte                   ' loop counter 
Dim MuteVal         As Word                   ' mute input
Dim Muted           As Bit                    ' muted = 1, open = 0  


                         '1705,2557,3410,4262,5115,5967,6820,7672,8525,9377,10230  ' 13 sWord points counting 0

Symbol CalON=PORTA.7                          ' calibrate button PB1
Symbol But1 = PORTA.6                         ' spare switch
Symbol Calib = 0                              ' store calibration in 1st eedata

'HD44780
DelayMS 10                                    ' Settle the display
Print $FE, $40                                ' HD44780 character program mode

Print $00, $00, $15, $15, $15, $15, $15, $00  ' Character 0: Three lines  |||
Print $00, $00, $10, $10, $10, $10, $10, $00  ' Character 1: One line     |
Print $00, $00, $14, $14, $14, $14, $14, $00  ' Character 2: Two lines    ||
Print $00, $00, $00, $00, $00, $00, $00, $00  ' Character 3: Scale         .

'Print $00, $00, $00, $15, $15, $15, $15, $15  ' Character 0: Three lines  |||
'Print $00, $00, $00, $10, $10, $10, $10, $10  ' Character 1: One line     |
'Print $00, $00, $00, $14, $14, $14, $14, $14  ' Character 2: Two lines    ||
'Print $00, $00, $00, $00, $00, $00, $00, $00  ' Character 3: Scale         .


'initialise things
SEGMENTS = 16                                 ' In this example we utilize a 16-segment display
Cls
CALIBRATED = ERead Calib                      ' flag for correction
Print At 1,1, "Deviation meter"
Print At 2,1, "by VK5VI & VK5DJ"
DelayMS 2000
Cls

GoTo Main
           
GetMeter:
           CountLoop = 0
           METER = 0
           For CountLoop = 1 To 50 
            DelayUS 20                       ' Charging time ADC
            METER = METER + ADIn 1           ' Pin 18 is ADC1 input @ 16F1827
           Next
           METER = METER/50 
Return

GetMute:
           DelayUS 20                       ' Charging time ADC
           MuteVal = ADIn 2                 ' Pin 1 is ADC2 input @ 16F1827
           If MuteVal > 205 Then            ' if more than 1V then mute open
              Muted = 0
           Else
              Muted = 1                     ' mute closed
           EndIf   
Return         


GetVolts:                                     'although gets volts, in readout its Dev Freq                                               
            PARTVOLTS = METER * 6
            VOLTREAD = PARTVOLTS / 1024
            PARTVOLTS = PARTVOLTS // 1024
            PARTVOLTS = ((PARTVOLTS * 10)+ 511) / 1024
            If PARTVOLTS > 9 Then              'roll it over if necessary
              Inc VOLTREAD
              PARTVOLTS = 0
            EndIf
            Print At 2,1,"Dev=", Dec VOLTREAD, "." , Dec PARTVOLTS, " kHz   "
Return

GetAdjustedFreq:                                 'goes here if calibrated = 1
            If METER < 115 Then 
                METER = 30
                Frequency = 0.0
            Else    
                Frequency = 6.6401 * METER - 395.23
                METER = Frequency/6000 * 1024      'for use by the graph section
                Frequency = Frequency / 1000
                Frequency = Frequency + 0.05
            EndIf
            If Frequency < 0.2 Then
               Print At 2,1, "Signal in noise "
            Else   
            Print At 2,1, "Dev=",Dec1 Frequency," kHz  *  "
            EndIf
Return 

'Do the calibration if button momentarily pressed            
DoCalibrateMode:             
               CountLoop = 0                   'set the loop counter
               While CountLoop < 100 And CalON = 0
                     Inc CountLoop
                     DelayMS 10
               Wend                            'if button held in for 2 secs then remove calibration
               If CountLoop > 99 Then  
                  If CALIBRATED = 0 Then       'flip calibration mode
                    CALIBRATED = 1
                  Else
                    CALIBRATED = 0
                  EndIf
                  Cls
                  If CALIBRATED = 1 Then 
                     Print At 1,1, "Calibrated      "
                  Else
                     Print At 1,1, "No calibration  " 
                  EndIf
                  DelayMS 2000         
                  If CALIBRATED > 0 Then 
                     EWrite Calib,[1]
                  Else 
                     EWrite Calib,[0]
                  EndIf   
               EndIf               
Return               

Main:
While 1 = 1                                   ' do it forever
            If CalON = 0 Then GoSub DoCalibrateMode ' if calibrate is on check for 1 sec and activate 
'Check the mute
            GoSub GetMute                     ' see if the mute is open

'ADC reading
            GoSub GetMeter 

            If CALIBRATED = 1 Then
               GoSub GetAdjustedFreq          ' use calibrations to improve readouts and print the freq on second line  
            Else
               GoSub GetVolts                 'raw number
            EndIf
            
'Calculate meter reading
            SUM = (METER * SEGMENTS) / 341    ' Meter value. Value 341 = 1024 / 3 (values per segment)
            FULL = SUM / 3                    ' Number of full characters |||
            PARTIAL = SUM // 3                ' Modulus restvalue (0, 1 or 2)


'Display the bargraph          
            Print At 1, 1, Rep 0\FULL         ' Displays full characters ||| of the S-meter
            If PARTIAL > 0 Then Print PARTIAL ' Prints either the empty, | or || character
            Print Rep 3\SEGMENTS - FULL       ' Clears the remaining part of the level bar          
            
'construct muted function            
 MuteOps:         If Muted = 1 Then
                  Print At 1,1, "     MUTED      "
                  If CALIBRATED = 1 Then
                     Print At 2,1, "  (calibrated)  "
                  Else   
                     Print At 2,1, "                "
                  EndIf
                  Repeat
                    GoSub GetMute
                    If CalON = 0 Then 
                        GoSub DoCalibrateMode
                        GoTo MuteOps
                    EndIf    
                  Until Muted = 0 
               Cls     
            EndIf   
                    
Wend
                
End
